@@ -4,7 +4,8 @@ from multiprocessing import Process, Event as event
 from multiprocessing.synchronize import Event
 from os import get_terminal_size
 from pathlib import Path
-from typing import Optional, Callable, Tuple, Any
+from traceback import print_exc
+from typing import Optional, Callable, Tuple, Any, Iterable
 from pprint import pformat
 
 import click
@@ -46,7 +47,7 @@ class MessagePrinter:
         if msg.contents['priority'] == 3:
             return 'red'
         if msg.contents['priority'] == 4:
-            return 'purple'
+            return 'magenta'
         return None
 
     @lru_cache
@@ -55,11 +56,13 @@ class MessagePrinter:
 
     def _header(self, msg: Message) -> str:
         width = self.term_width
-        return f"{'=' * ((width - 2 - len(self._label(msg))) // 2 + 1)} {self._label(msg)} {'=' * ((width - 2 - len(self._label(msg))) // 2 - (1 if len(self._label(msg)) % 2 == 0 else 0))}"
+        untrimmed_header = f"{'=' * ((width - 2 - len(self._label(msg))) // 2 + 1)} {self._label(msg)} {'=' * ((width - 2 - len(self._label(msg))) // 2)}"
+        return untrimmed_header[:width]
 
     def _route(self, msg: Message) -> str:
         width = self.term_width
-        return f"{'-' * ((width - 2 - len(msg.routing_key)) // 2 + 1)} {msg.routing_key} {'-' * ((width - 2 - len(msg.routing_key)) // 2 - (1 if len(msg.routing_key) % 2 == 0 else 0))}"
+        untrimmed_route = f"{'-' * ((width - 2 - len(msg.routing_key)) // 2 + 1)} {msg.routing_key} {'-' * ((width - 2 - len(msg.routing_key)) // 2)}"
+        return untrimmed_route[:width]
 
     @typechecked
     def watch(self, msg: Message) -> None:
@@ -74,28 +77,46 @@ class MessagePrinter:
 
 
 @typechecked
-def _watch_process(exchange: str,
-                   routing_key: str,
-                   msg_limit_received: Optional[Event] = None,
+def _watch_process(exchanges: Iterable[str],
+                   exchange_bindings: Iterable[Tuple[str, str]],
+                   alert_exchanges: Iterable[str],
+                   done: Event,
                    config: Optional[RobotnikConfig] = None,
                    num_msgs: Optional[int] = None):
-    watcher = MessagePrinter(msg_limit_received, num_msgs)
-    sub = Subscriber(exchange=exchange, binding_key=routing_key, config=config)
-    sub.register_listener(watcher.watch)
-    sub.run()
+    try:
+        watcher = MessagePrinter(done, num_msgs)
+        sub = Subscriber(config=config)
+        for exchange in exchanges:
+            sub.bind(exchange)
+        for exchange, binding in exchange_bindings:
+            sub.bind(exchange, binding)
+        for exchange in alert_exchanges:
+            sub.bind(exchange, '#.warning')
+            sub.bind(exchange, '#.error')
+            sub.bind(exchange, '#.critical')
+        sub.run(watcher.watch)
+    except KeyboardInterrupt:
+        pass
+    except Exception:  # pylint: disable=W0703
+        print_exc()
+    finally:
+        done.set()
 
 
 @typechecked
-def _watch(exchange: str, routing_key: str,
+def _watch(exchanges: Iterable[str],
+           exchange_bindings: Iterable[Tuple[str, str]],
+           alert_exchanges: Iterable[str],
            num_msgs: Optional[int] = None,
            config: Optional[RobotnikConfig] = None):
-    msg_limit_received = event()
-    with timeout_process(target=_watch_process, args=(exchange,
-                                                      routing_key,
-                                                      msg_limit_received,
+    done = event()
+    with timeout_process(target=_watch_process, args=(exchanges,
+                                                      exchange_bindings,
+                                                      alert_exchanges,
+                                                      done,
                                                       config,
                                                       num_msgs)):
-        msg_limit_received.wait()
+        done.wait()
 
 
 @click.group()
@@ -103,16 +124,21 @@ def cli():
     """A set of utilities for working with legion on the commandline"""
 
 
-@cli.command()  # @cli, not @click!
-@click.argument('exchange')
-@click.option('-r', '--routing-key', default='#',
-              help='The routing key which is used to filter messages on RobotnikMQ '
-                   '(RabbitMQ) Topic exchanges. By default, will be set to # which'
-                   ' will output all messages on the exchange.')
+@cli.command()
+@click.option('-e', '--exchange', 'exchanges', type=str, multiple=True, default=[],
+              help='An exchange to subscribe to (uses the # binding key which returns all messages on the exchange). Can be used multiple times.')
+@click.option('-r', '--exchange-route', 'exchange_routes', type=click.Tuple([str, str]), nargs=2,
+              multiple=True, default=[],
+              help='An exchange and routing key combination which allows us to filter messages on the exchange. Can be used multiple times. See https://www.rabbitmq.com/tutorials/tutorial-five-python.html for more information on routing keys.')
+@click.option('-a', '--alerts', 'alert_exchanges', type=str, multiple=True, default=[],
+              help='Monitor an exchange for all alerts (warning,error,critical). Equivalent to `-r <EXCHANGE> #.warning -r <EXCHANGE> #.error -r <EXCHANGE> #.critical`')
 @click.option('-n', '--msg-limit', default=None, type=int,
               help='If set, this will cause the script to finish after a given number of messages '
                    'has been received.')
-def watch(exchange: str, routing_key: str, msg_limit):
+def watch(exchanges: Iterable[str],
+          exchange_routes: Iterable[Tuple[str, str]],
+          alert_exchanges: Iterable[str],
+          msg_limit: Optional[int]):
     """Given an exchange name, this utility will monitor all messages going through said
        exchange (subject to optional filters) and output them to STDOUT."""
-    _watch(exchange, routing_key, msg_limit)
+    _watch(exchanges, exchange_routes, alert_exchanges, msg_limit)
